"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.calculators = void 0;
function _evaluators() {
  const data = require("@nocobase/evaluators");
  _evaluators = function _evaluators() {
    return data;
  };
  return data;
}
function _utils() {
  const data = require("@nocobase/utils");
  _utils = function _utils() {
    return data;
  };
  return data;
}
var _constants = require("../constants");
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const calculators = new (_utils().Registry)();
// built-in functions
exports.calculators = calculators;
function equal(a, b) {
  return a === b;
}
function notEqual(a, b) {
  return a !== b;
}
function gt(a, b) {
  return a > b;
}
function gte(a, b) {
  return a >= b;
}
function lt(a, b) {
  return a < b;
}
function lte(a, b) {
  return a <= b;
}
calculators.register('equal', equal);
calculators.register('notEqual', notEqual);
calculators.register('gt', gt);
calculators.register('gte', gte);
calculators.register('lt', lt);
calculators.register('lte', lte);
calculators.register('===', equal);
calculators.register('!==', notEqual);
calculators.register('>', gt);
calculators.register('>=', gte);
calculators.register('<', lt);
calculators.register('<=', lte);
function includes(a, b) {
  return a.includes(b);
}
function notIncludes(a, b) {
  return !a.includes(b);
}
function startsWith(a, b) {
  return a.startsWith(b);
}
function notStartsWith(a, b) {
  return !a.startsWith(b);
}
function endsWith(a, b) {
  return a.endsWith(b);
}
function notEndsWith(a, b) {
  return !a.endsWith(b);
}
calculators.register('includes', includes);
calculators.register('notIncludes', notIncludes);
calculators.register('startsWith', startsWith);
calculators.register('notStartsWith', notStartsWith);
calculators.register('endsWith', endsWith);
calculators.register('notEndsWith', notEndsWith);
function calculate(calculation = {}) {
  var _calculation$operands;
  let fn;
  if (!(calculation.calculator && (fn = calculators.get(calculation.calculator)))) {
    throw new Error(`no calculator function registered for "${calculation.calculator}"`);
  }
  return Boolean(fn(...((_calculation$operands = calculation.operands) !== null && _calculation$operands !== void 0 ? _calculation$operands : [])));
}
function logicCalculate(calculation) {
  if (!calculation) {
    return true;
  }
  if (typeof calculation['group'] === 'object') {
    var _calculation$group$ca;
    const method = calculation['group'].type === 'and' ? 'every' : 'some';
    return ((_calculation$group$ca = calculation['group'].calculations) !== null && _calculation$group$ca !== void 0 ? _calculation$group$ca : [])[method](item => logicCalculate(item));
  }
  return calculate(calculation);
}
var _default = {
  run(node, prevJob, processor) {
    return _asyncToGenerator(function* () {
      const _ref = node.config || {},
        engine = _ref.engine,
        calculation = _ref.calculation,
        expression = _ref.expression,
        rejectOnFalse = _ref.rejectOnFalse;
      const evaluator = _evaluators().evaluators.get(engine);
      let result = true;
      try {
        result = evaluator ? evaluator(expression, processor.getScope()) : logicCalculate(processor.getParsedValue(calculation, node));
      } catch (e) {
        return {
          result: e.toString(),
          status: _constants.JOB_STATUS.ERROR
        };
      }
      if (!result && rejectOnFalse) {
        return {
          status: _constants.JOB_STATUS.FAILED,
          result
        };
      }
      const job = {
        status: _constants.JOB_STATUS.RESOLVED,
        result,
        // TODO(optimize): try unify the building of job
        nodeId: node.id,
        upstreamId: prevJob && prevJob.id || null
      };
      const branchNode = processor.nodes.find(item => item.upstream === node && Boolean(item.branchIndex) === result);
      if (!branchNode) {
        return job;
      }
      const savedJob = yield processor.saveJob(job);
      return processor.run(branchNode, savedJob);
    })();
  },
  resume(node, branchJob, processor) {
    return _asyncToGenerator(function* () {
      if (branchJob.status === _constants.JOB_STATUS.RESOLVED) {
        // return to continue node.downstream
        return branchJob;
      }
      // pass control to upper scope by ending current scope
      return processor.end(node, branchJob);
    })();
  }
};
exports.default = _default;